%% Target to Target Transmission using UDP
% Transmit data between two target PCs using UDP blocks.
%
% This example demonstrates how to use UDP blocks to send data between two
% target PCs. The model |struture_xpc1.mdl| runs on TargetPC1.  The
% model |damper_pc2.mdl| runs on TargetPC2.  Both models send and receive
% signal data packets.
%
% The model |struture_xpc1.mdl| simulate a 2-story of frame sutrcture, send
% 1st floor displacement, velocity from Target1 to Target2
%
% The model |damper_pc2.mdl| simulate a MR damper located between the
% ground and 1st floor of that structure. It get the 1st floor
% displacement, velocity from Target1 and send the damper force back to
% Target1.
%
%%
% *Note:* When considering UDP as a protocol for communicating data to/from the 
% xPC Target(TM) environment it is important to be aware of the following:
% 
% * UDP is a _connectionless_ protocol with no checks to confirm successful
% transmission of packets.  Hence, data packets can be lost or dropped. 
% * On the target PC, UDP blocks run in a background task which executes each
% time step after the real-time task completes. If there is not enough time to
% run or complete the background task before the next time step, data may not be
% communicated. 
% * UDP data packets are transmitted over the host-target Ethernet link and must
% therefore share bandwidth with host-target communications.
% * For more information on using UDP with xPC Target see the Help documentation section:
% <matlab:helpview(fullfile(docroot,'mapfiles','xpc.map'),'UDP') UDP I/O Support>
%   
%
%    Written by: Xin Li, Tongji University, August 2013
%                with support from Sarah Hacker and 
%                Shirley J. Dyke, Purdue University 


%% Load parameters to workspace
clc; clear all; close all

parameters

% sim set
fs = 1000;              % sampling rate
sim_length = 20;        % seconds 

dtint   = 0.0001;		% Integration Time Step
dec  	  = 10;			% Decimation Step
%% Open, Build, and Download the TargetPC1 Model.
mdlOpen = 0;
systems = find_system('type', 'block_diagram');
if isempty(strmatch('structure_xpc1', systems, 'exact'))
  mdlOpen = 1;
  open_system('structure_xpc1');  
end

%% Build the model and download to the target PC.
set_param('structure_xpc1','RTWVerbose','off'); % Configure for a non-Verbose build.
rtwbuild('structure_xpc1');                     % Build and download application.

%%

% Close the model if we opened it.
if (mdlOpen)
  bdclose('structure_xpc1');
end

%% Open, Build, and Download the TargetPC2 Model

% Open the model.
mdlOpen = 0;
systems = find_system('type', 'block_diagram');
if isempty(strmatch('damper_xpc2', systems, 'exact'))
  mdlOpen = 1;
  open_system('damper_xpc2');  
end

%%

% Build the model and download to the target PC.
set_param('damper_xpc2','RTWVerbose','off'); % Configure for a non-Verbose build.
rtwbuild('damper_xpc2');                     % Build and download application.

%%

% Close the model if we opened it.
if (mdlOpen)
  bdclose('damper_xpc2');
end

%% Run both Models
% Using the xPC Target object variables |tg1| and |tg2|, start the models.
pause(5);   % Run for 5 seconds.
% Start the TargetPC1 and TargetPC2 model.
start(tg2);
pause(2);   % xPC2 Run for 2 seconds.
start(tg1);
pause(sim_length);   % Run for sim_length seconds.
%% Stop both Models
% Using the xPC Target object variables |tg1| and |tg2|, stop the models.

stop(tg1); % Stop the TargetPC1 model.
stop(tg2); % Stop the TargetPC2 model.
displayEndOfDemoMessage(mfilename)

TargetIP = '128.46.160.52';         % Change this to the IP of the On-board network IP of target 1
target_ftp = xpctarget.ftp('TCPIP',TargetIP,'22222');

% Pull data from damper force simulation into workspace 
get(target_ftp,'tg1_sq2.dat')
Received_signal = readxpcfile('tg1_sq2.dat');

time_r = Received_signal.data(:,end);
force = Received_signal.data(:,1:end-1); %damper force

% Pull data from structure simulation into workspace 
get(target_ftp,'tg1_sq3.dat')
response = readxpcfile('tg1_sq3.dat');
x1 = response.data(:,1); % 1st floor displacement
x2 = response.data(:,2); % 2nd floor displacement
v1 = response.data(:,3); % 1st floor veloity
v2 = response.data(:,4); % 2nd floor veloity
a1 = response.data(:,5); % 1st floor acceleration
a2 = response.data(:,6); % 2nd floor acceleration

% Save all data from RTHS into a file 
save SimOutputs time_r x1 x2 v1 v2 a1 a2 force

% PLOT RESULTS OF RTHS 
subplot(311)
plot(time_r, [x1 x2],'LineWidth',2)
grid on;
legend('1st floor displacemnet','2nd floor displacement')
xlabel('time(s)');ylabel('displacement(mm)');
title('Displacement Response')

subplot(312)
plot(time_r, [v1 v2],'LineWidth',2)
grid on;
legend('1st floor velocity','2nd floor velocity')
xlabel('time(s)');ylabel('velocity(mm/s)');
title('Velocity Response')

subplot(313)
plot(time_r, force,'LineWidth',2)
legend('MR damper force')
xlabel('time(s)');ylabel('force(N)');
grid on;
title('Force Response')
